<?php

namespace WilokeProductSaleCountDown\Dashboard\Controllers;


use Exception;
use WilokeProductSaleCountDown\Dashboard\Shared\App;
use WilokeProductSaleCountDown\Dashboard\Shared\GeneralHelper;
use WilokeProductSaleCountDown\Dashboard\Shared\Option;
use WilokeProductSaleCountDown\Illuminate\Prefix\AutoPrefix;


class DashboardController
{
	use GeneralHelper;

	const WILSCD_GLOBAL = 'WILSCD_GLOBAL';
	public array $aOptions = [];

	public function __construct()
	{
		App::bind('style', include plugin_dir_path(__FILE__) . '../Configs/Style.php');
		add_action('admin_menu', [$this, 'registerMenu']);
		add_action('admin_enqueue_scripts', [$this, 'enqueueScriptsToDashboard']);
		add_action('wp_enqueue_scripts', [$this, 'enqueueScripts'], 200);
		add_action('init', [$this, 'do_output_buffer']);
	}
	function do_output_buffer() {
		ob_start();
	}
	/**
	 * @throws Exception
	 */
	public function enqueueScripts()
	{
		$position = '';
		$productId = get_the_ID();
		if (!empty($productId)) {
			$position = get_post_meta(
				$productId,
				AutoPrefix::namePrefix('position-product-count-down'),
				true
			) ?: 'bottomTitle';
		}
		wp_localize_script('jquery', self::WILSCD_GLOBAL, [
			'Days'     => esc_html__('days', 'wiloke-product-sale-countdown'),
			'Hrs'      => esc_html__('hrs', 'wiloke-product-sale-countdown'),
			'Mins'     => esc_html__('mins', 'wiloke-product-sale-countdown'),
			'Secs'     => esc_html__('secs', 'wiloke-product-sale-countdown'),
			'position' => $position,
			'colorStyle' =>  Option::getAuthSettings()['colorStyle'],
			'content'  => (!empty($position) && ($position == 'bottomThumbnail')) ?
				apply_filters('wil-pscd-get-content',
					$productId) : ''
		]);

		wp_enqueue_script(
			uniqid(AutoPrefix::namePrefix('wiloke-countdown')),
			plugin_dir_url(__FILE__) . '../Assets/Js/wiloke.countdown.js',
			['jquery'],
			WILOKE_PSCD_VERSION,
			true
		);
		wp_enqueue_script(
			uniqid(AutoPrefix::namePrefix('dashboard-script')),
			plugin_dir_url(__FILE__) . '../Assets/Js/Script.js',
			['jquery'],
			WILOKE_PSCD_VERSION,
			true
		);
		wp_enqueue_style(
			uniqid(AutoPrefix::namePrefix('dashboard-style')),
			plugin_dir_url(__FILE__) . '../Assets/Css/Style.css',
			[],
			WILOKE_PSCD_VERSION
		);
	}

	public function enqueueScriptsToDashboard($hook): bool
	{
		if (strpos($hook, $this->getDashboardSlug()) !== false) {

			wp_localize_script('jquery', self::WILSCD_GLOBAL, [
				'colorStyle' => Option::getOptionStyleColor()
			]);
			wp_enqueue_script('wp-color-picker');
			wp_enqueue_script(
				uniqid(AutoPrefix::namePrefix('dashboard-script-admin')),
				plugin_dir_url(__FILE__) . '../Assets/Js/AdminScript.js',
				['jquery'],
				WILOKE_PSCD_VERSION,
				true
			);

			wp_enqueue_style('wp-color-picker');
			wp_enqueue_style(
				uniqid(AutoPrefix::namePrefix('dashboard-style')),
				plugin_dir_url(__FILE__) . '../Assets/Css/AdminStyle.css',
				[],
				WILOKE_PSCD_VERSION
			);
		}
		return false;
	}


	public function registerMenu()
	{
		add_menu_page(
			esc_html__('Product Sale Countdown Dashboard', 'wiloke-product-sale-countdown'),
			esc_html__('Product Sale Countdown Dashboard', 'wiloke-product-sale-countdown'),
			'administrator',
			$this->getDashboardSlug(),
			[$this, 'renderSettings'],
			'dashicons-clock'
		);
	}

	public function saveOption()
	{
		$aValues = [];
		if (isset($_POST['wilpscd-auth-field']) && !empty($_POST['wilpscd-auth-field'])) {
			if (wp_verify_nonce($_POST['wilpscd-auth-field'], 'wilpscd-auth-action')) {
				if (isset($_POST['wilpscd-cb']) && !empty($_POST['wilpscd-cb'])) {
					foreach ($_POST['wilpscd-cb'] as $key => $val) {
						if ($key == 'style') {
							$aValues['colorStyle'] =Option::getOptionStyleColor();
							$aValues['colorStyle'][$val] = [
									'numberColor'   => $_POST['wilpscd-cb']['numberColor'] ?? '',
									'numberBgColor' => $_POST['wilpscd-cb']['numberBgColor'] ?? '',
									'textColor'     => $_POST['wilpscd-cb']['textColor'] ?? '',
									'textBgColor'   => $_POST['wilpscd-cb']['textBgColor'] ?? ''
							];
						}
						$aValues[sanitize_text_field($key)] = sanitize_text_field(trim($val));
					}
				}
				Option::saveAuthSettings($aValues);
				wp_redirect($_SERVER['HTTP_REFERER']);
			}
		}

	}
	public function renderSettings()
	{
		$this->saveOption();
		$this->aOptions = Option::getAuthSettings();

		include plugin_dir_path(__FILE__) . '../Views/DashboardSettings.php';
	}

}
