<?php

namespace WilokeProductSaleCountDown\Dashboard\Shared;

class Option
{
	private static string $optionKey = WILOKE_PSCD_PREFIX . 'product-sale-count-down';

	private static array $aDataOptions = [];

	public static function saveAuthSettings(array $aValues)
	{
		update_option(self::$optionKey, $aValues);
	}

	public static function deleteAuthSettings()
	{
		delete_option(self::$optionKey);
	}

	public static function defaultColorStyle(): array
	{
		return [
			'style1' => [
				'numberColor'   => '#111',
				'numberBgColor' => '#eee',
				'textColor'     => '#eee',
				'textBgColor'   => '#111'
			],
			'style2' => [
				'numberColor'   => '#8659ef',
				'numberBgColor' => '#eee',
				'textColor'     => '#666',
				'textBgColor'   => 'transparent'
			],
			'style3' => [
				'numberColor'   => '#fff',
				'numberBgColor' => '#ef6d59',
				'textColor'     => '#fff',
				'textBgColor'   => 'transparent'
			],
			'style4' => [
				'numberColor'   => '#fff',
				'numberBgColor' => '#34cd69',
				'textColor'     => '#fff',
				'textBgColor'   => 'transparent'
			],
		];
	}

	public static function getTitleDefault()
	{
		return self::getFields('title', '');
	}

	public static function getOptionStyleColor()
	{
		return self::getFields('colorStyle', '') ?: self::defaultColorStyle();
	}

	public static function getFields($field, $default = '')
	{
		self::getAuthSettings();
		return self::$aDataOptions[$field] ?? $default;
	}

	public static function getAuthSettings()
	{
		self::$aDataOptions = get_option(self::$optionKey) ?: [];
		return self::$aDataOptions;
	}

	public static function getStyleDefault()
	{
		return self::getFields('style', '');
	}

	public static function getNumberColorDefault()
	{
		return self::getFields('numberColor', '');
	}

	public static function getNumberBackgroundColorDefault()
	{
		if (self::getStyleDefault() == 'style-1') {
			return self::getFields('numberBgColor', '');
		}
		return 'transparent';
	}

	public static function getTextColorDefault()
	{
		return self::getFields('textColor', '');
	}

	public static function getTitleCenter()
	{
		return self::getFields('titleCenter', '');
	}

	public static function getTextSizeTitle()
	{
		return self::getFields('textSizeTitle', '');
	}

	public static function getTextSizeScd1()
	{
		return self::getFields('textSizeScd1', '');
	}

	public static function getTextSizeScd2()
	{
		return self::getFields('textSizeScd2', '');
	}

	public static function getTextBackgroundColor(): string
	{
		$data = '';
		$textBgColor = self::getFields('textBgColor', '');
		switch (self::getStyleDefault()) {
			case 'style1':
				$data = '--text-bg-color:' . $textBgColor . ';';
				break;
			case 'style2':
			case 'style3':
			case 'style4':
				$data = ' --text-bg-color: transparent;';
				break;
		}
		return $data;
	}
}
