<?php

namespace WilokeProductSaleCountDown\Product\Controllers;

use DateTime;
use DateTimeZone;
use Exception;
use WilokeProductSaleCountDown\Dashboard\Shared\App;
use WilokeProductSaleCountDown\Dashboard\Shared\Option;
use WilokeProductSaleCountDown\Dashboard\Shared\TraitGetSettingProductCountDown;
use WilokeProductSaleCountDown\Illuminate\Prefix\AutoPrefix;

class ProductController
{
	use TraitGetSettingProductCountDown;

	private static ?ProductController $oClassSingleton = null;

	public function __construct()
	{
		add_filter('woocommerce_single_product_image_gallery_classes', [$this, 'addClassProductGallery'], 10);
		add_filter('wil-pscd-get-content', [$this, 'getContentGalleryImage']);
		add_action('woocommerce_single_product_summary', [$this, 'handleDisplayCountDownBottomTitle'], 10);
		add_action('woocommerce_product_options_general_product_data', [$this, 'handleAddMetaDataProductSale']);
		add_action('woocommerce_process_product_meta', [$this, 'handleSaveProductMeta'], 10, 2);
		add_action('init', [$this, 'loadShopLoopItem'], 999);
	}

	public function loadShopLoopItem()
	{
		add_action('woocommerce_after_shop_loop_item', [$this, 'handleDisplayCountDownCategory'], 5);
	}

	public function getPositionCD()
	{
		return get_post_meta(get_the_ID(), AutoPrefix::namePrefix('position-product-count-down'), true);
	}

	/**
	 * @throws Exception
	 */
	public function addClassProductGallery($aClass)
	{
		if ($this->getPositionCD() == 'bottomThumbnail') {
			$aClass[] = 'wil-sale-countdown-gallery';
		}
		return $aClass;
	}

	function handleSaveProductMeta($id, $post)
	{
		if (isset($_POST[AutoPrefix::namePrefix('title-product-count-down')])) {
			update_post_meta($id, AutoPrefix::namePrefix('title-product-count-down'),
				$_POST[AutoPrefix::namePrefix('title-product-count-down')]);
		}
		if (isset($_POST[AutoPrefix::namePrefix('style-product-count-down')])) {
			update_post_meta($id, AutoPrefix::namePrefix('style-product-count-down'),
				$_POST[AutoPrefix::namePrefix('style-product-count-down')]);
		}
		if (isset($_POST[AutoPrefix::namePrefix('position-product-count-down')])) {
			update_post_meta($id, AutoPrefix::namePrefix('position-product-count-down'),
				$_POST[AutoPrefix::namePrefix('position-product-count-down')]);
		}
	}

	public function handleAddMetaDataProductSale()
	{
		$productID = get_the_ID();
		$salePriceDateTo = (int)get_post_meta($productID, '_sale_price_dates_to', true);
		if (!empty($salePriceDateTo)) {
			echo '<div class="options_group">';

			woocommerce_wp_textarea_input([
				'id'          => AutoPrefix::namePrefix('title-product-count-down'),
				'value'       => get_post_meta(get_the_ID(), AutoPrefix::namePrefix('title-product-count-down'),
					true) ?:
					'Sale ends soon!',
				'label'       => esc_html__('Title Product Sale Countdown', 'wiloke-product-sale-countdown'),
				'desc_tip'    => true,
				'description' => esc_html__('Title Product Sale Countdown', 'wiloke-product-sale-countdown'),
			]);

			woocommerce_wp_select([
				'id'          => AutoPrefix::namePrefix('style-product-count-down'),
				'value'       => get_post_meta(get_the_ID(), AutoPrefix::namePrefix('style-product-count-down'),
					true) ?: 'style3',
				'label'       => esc_html__('Style Product Sale Countdown', 'wiloke-product-sale-countdown'),
				'desc_tip'    => true,
				'options'     => array_merge(['inheritDefault' => 'Inherit Default'], App::get('style')),
				'description' => esc_html__('Style Product Sale Countdown', 'wiloke-product-sale-countdown'),
			]);

			woocommerce_wp_select([
				'id'          => AutoPrefix::namePrefix('position-product-count-down'),
				'value'       => $this->getPositionCountDown(get_the_ID()),
				'label'       => esc_html__('Add The Sale Countdown position', 'wiloke-product-sale-countdown'),
				'desc_tip'    => true,
				'options'     => [
					'bottomThumbnail' => esc_html__('Bottom Image Thumbnail',
						'wiloke-product-sale-countdown'),
					'bottomTitle'     => esc_html__('Bottom Title Product',
						'wiloke-product-sale-countdown')
				],
				'description' => esc_html__('Where do you want to add the Sale Countdown on the Single Product page?',
					'wiloke-product-sale-countdown')
			]);

			echo '</div>';
		}
	}

	public function getPositionCountDown(int $productID)
	{
		return get_post_meta($productID, AutoPrefix::namePrefix('position-product-count-down'), true) ?: 'bottomTitle';
	}

	/**
	 * @throws Exception
	 */
	public function handleDisplayCountDownBottomTitle()
	{
		if (self::registerClassSingleton()->registerClassSingleton()->isDisplayBottomTitle()) {
			echo self::registerClassSingleton()->registerClassSingleton()->handleCheckProductOnSaleSchedule();
		}
	}

	public function isDisplayBottomTitle(): bool
	{
		$position = get_post_meta(get_the_ID(), AutoPrefix::namePrefix('position-product-count-down'), true);
		if (!empty($position)) {
			return $position == 'bottomTitle';
		}
		return true;
	}

	public static function registerClassSingleton(): ProductController
	{
		if (self::$oClassSingleton == null) {
			self::$oClassSingleton = App::get('oProduct');
		}
		return self::$oClassSingleton;
	}

	/**
	 * @throws Exception
	 */
	public function handleCheckProductOnSaleSchedule()
	{
		$productID = get_the_ID();
		$salePriceDateTo = (int)get_post_meta($productID, '_sale_price_dates_to', true);
		$content = '';
		if (empty($salePriceDateTo)) {
			global $product;
			if (is_a($product, 'WC_Product')) {
				if ($product->is_type('variable')) {
					$variationIds = $product->get_visible_children();
					$aSalePriceDateTo = [];
					foreach ($variationIds as $variationId) {
						$oVariation = wc_get_product($variationId);
						if ($oVariation->is_on_sale()) {
							$oDateOnSaleTo = $oVariation->get_date_on_sale_to();
							if (!empty($oDateOnSaleTo)) {
								$aSalePriceDateTo[] = $oDateOnSaleTo->getTimestamp();
							}
						}
					}
					if (!empty($aSalePriceDateTo)) {
						rsort($aSalePriceDateTo);
						$salePriceDateTo = ($aSalePriceDateTo[0]);
					}
				}
			}
		}
		$timezoneDate = wp_timezone_string();
		$oDate = new DateTime('now', new DateTimeZone($timezoneDate));
		$oDate->setTimestamp($salePriceDateTo);
		$dateDayCountdown = $oDate->getTimestamp();
		$dateDayCurrent = strtotime(date('Y/m/d', time()));
		$dataCountdown = $oDate->format('Y/m/d');
		if (!empty($salePriceDateTo) && ($dateDayCountdown > $dateDayCurrent) &&
			($dataCountdown != date('Y/m/d', time()))) {
			$titleCountdown = $this->getTitleProductCountDown($productID);
			$styleCountdown = $this->getStyleProductCountDown($productID);
            $optionStyleColor = Option::getOptionStyleColor();
            $numColor =$optionStyleColor[$styleCountdown]['numberColor'];
            $numberBgColor =$optionStyleColor[$styleCountdown]['numberBgColor'];
            $textBgColor =$optionStyleColor[$styleCountdown]['textBgColor'];
            $textColor =$optionStyleColor[$styleCountdown]['textColor'];
			ob_start();
			?>
            <div class="wil-scd-container" style="
                    --align: <?php echo esc_attr(Option::getTitleCenter()); ?>;
                    --font-size-title: <?php echo esc_attr(Option::getTextSizeTitle()); ?>;
                    ">
                <p class="wil-scd-desc"><?php echo esc_html($titleCountdown); ?></p>
                <div data-countdown="<?php echo esc_attr($dataCountdown); ?>"
                     class="wil-scd-control <?php echo esc_attr($styleCountdown); ?>" style="
                        --num-color: <?php echo esc_attr($numColor); ?>;
                        --num-bg-color: <?php echo esc_attr($numberBgColor); ?>;
                        --text-color: <?php echo esc_attr($textColor); ?>;
                        --text-bg-color: <?php echo esc_attr($textBgColor); ?>;
                        --font-size1: <?php echo esc_attr(Option::getTextSizeScd1()); ?>;
                        --font-size2: <?php echo esc_attr(Option::getTextSizeScd2()); ?>;
                        --align: <?php echo esc_attr(Option::getTitleCenter()); ?>;
				<?php echo esc_attr(Option::getTextBackgroundColor()); ?>">
                </div>
            </div>
            <br>
			<?php
			$content = ob_get_contents();
			ob_end_clean();
		}
		return $content;
	}

	/**
	 * @throws Exception
	 */
	public function handleDisplayCountDownBottomThumbnails()
	{
		if (self::registerClassSingleton()->getPositionCD() == 'bottomThumbnail') {
			echo self::registerClassSingleton()->handleCheckProductOnSaleSchedule();
		}
	}

	/**
	 * @throws Exception
	 */
	public function getContentGalleryImage($productID)
	{
		return self::registerClassSingleton()->handleCheckProductOnSaleSchedule();
	}

	/**
	 * @throws Exception
	 */
	public function handleDisplayCountDownCategory()
	{
		echo self::registerClassSingleton()->handleCheckProductOnSaleSchedule();
	}
}
